; =======================================================================================================
; PutSprite16Masked : Draw a 16*16 masked sprite everywhere on the screen
; by Thibaut Chevalier (203 bytes)
; in:  hl = start of the 16*16 masked sprite (the two first byte must be height and width)
;      b = x (0,127) and c = y (0,63)
; out: Sprite drawn
;      All registers destroyed
; =======================================================================================================

PutSprite16Masked:
    push bc                 ; save the position of the sprite
    ld c, b                 ; we temporary save b (x position) into c
    inc hl
    inc hl                  ; add two with hl to get the real start of the sprite data (these numbers are practical for the sprite editor, i think)
    ld de, virtual_sprite   ; first, i'm going to copy your sprite in virtual_sprite, which is a 24*16 masked temporary sprite
    ex de, hl               ; de point at the sprite adress, hl point at virtual_sprite adress
    ld b, 32                ; with djnz, we're now ready for 32 loop
Mcopy_loop:
    ld a, (de)              ; get the first byte of the row in a
    ld (hl), a              ; save it in the first byte at the same row in virtual_sprite
    inc de                  ; ready to read the next byte in the sprite
    inc hl                  ; ready to write the second byte
    ld a, (de)              ; get the second byte of the row in a
    ld (hl), a              ; save it in the second byte at the same row in virtual_sprite
    inc de                  ; de point to the next row of your sprite
    inc hl                  ; ready to write the third byte of virtual_sprite
    ld (hl), 0              ; reset the third byte of the row
    inc hl                  ; hl point to the next row of virtual_sprite : Remember, virtual_sprite is a 24*16 sprite (3 bytes large) 
    djnz Mcopy_loop         ; dec b and if b=0, then the sprite is finished copied (16 rows of data and 16 rows of mask) 
    ld a, c                 ; c is the x position
    and %00000111           ; get in a the colomn of the sprite on the grid
                            ; now, we're gonna slide virtual_sprite a time on the right
    jr z, Mno_slide         ; if a=0, we don't modify virtual_sprite
    ld b, a                 ; b is the number of 1-bit slides
Mslide_loop2:
    ld c, b                 ; now c is the number of 1-bit slides: this instruction save b in c before each slide (because b is destroyed during it)
    ld b, 32                ; we have 32 rows to slide
    ld hl, virtual_sprite
Mslide_loop1:
    or a                    ; now carry flag is reset (we inser 0 on the left)
    rr (hl)                 ; first byte of the row is rotated 1-bit position through the carry flag
    push af                 ; save the content of the carry flag
    inc hl
    pop af                  ; load the previous content of the carry flag (we inser bit-0 of the first byte)
    rr (hl)                 ; second byte of the row is rotated 1-bit position through the carry flag
    push af                 ; save the content of the carry flag
    inc hl
    pop af                  ; load the previous content of the carry flag (we inser bit-0 of the second byte)
    rr (hl)                 ; third byte of the row is rotated 1-bit position through the carry flag
                            ; now one row has been slided one time
    inc hl                  ; hl point at the next row
    djnz Mslide_loop1       ; dec b and if b=0, then virtual_sprite has been slided right 1-bit position (16 rows of data and 16 rows of mask) 
    ld b, c                 ; get the number of slides that have been done (saved in c)
    djnz Mslide_loop2
Mno_slide:
    pop bc                  ; restore the initials values of b and c
    ld d, c
    ld e, b                 ; Findpixel need e = x and d = y
    call FindPixel          ; now hl is the start of diplay memory
    ld de, virtual_sprite   ;     de is the start of the sprite we prepare just before
MRestore_mid:
    ld b, 16                ; we have 16 rows to draw (each 3 bytes large)
Mdraw_loop:
    push bc                 ; save counter (b) value
    call Mmask_loop         ; draw the byte of the current adress to the screen with masking
    inc hl                  ; hl point to the next byte on the screen
    inc de                  ; de point to the second byte of the current row of virtual_sprite
    call Mmask_loop         ; draw the byte of the current adress to the screen with masking
    inc hl                  ; hl point to the next byte on the screen
    inc de                  ; de point to the third byte of the current row of virtual_sprite
    call Mmask_loop         ; draw the byte of the current adress to the screen with masking
    ld bc, 14               ; load bc for a 16-bit addition
    add hl, bc              ; hl point to the first row of the next colomn (on the screen)
    inc de                  ; de too (for the sprite, we just have to increment de)
    pop bc                  ; restore the counter (b) value for 'djnz'
    djnz Mdraw_loop         ; execute the loop 16 times, each time for one row
    ret                     ; it's all over !! (all registers are destroyed)
Mmask_loop:
    ld a, (de)              ; get byte to draw in register a
    or (hl)                 ; set pixels without destroying the background
    ld (hl), a              ; save changes to screen
    ex de, hl               ; de = display adress / hl = data adress
    ld bc, 48               ; 48 is what we need to add to the current sprite adress to point at the corresponding mask
    add hl, bc              ; hl point to mask
    ld a, (hl)              ; a = mask
    scf                     ; set carry flag
    ccf                     ; invert carry flag, now carry flag is reset (this instruction doesn't exist!)
    sbc hl, bc              ; hl retuer to the data adress (in sprite memory)
    ld b, (hl)              ; b = data, a = mask
    xor b                   ; pixels we want to be off are set in register a 
    cpl                     ; now they are reset and the others set
    ex de, hl               ; de = data adress / hl = display adress      
    and (hl)                ; reset pixels without destroying the background
    ld (hl), a              ; save changes to screen
    ret                     ; return to the calling part of routine
virtual_sprite:             ; virtual_sprite is a 24*16 masked temporary sprite
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0

; =======================================================================================================
; GetBack16 : Get a 16*16 sprite from the screen into background_sprite (188 bytes)
; by Thibaut Chevalier
; in:  b = x (0,127) and c = y (0,63)
; out: background_sprite in a sprite that countain the 16*16 area beginning at (b, c) on the screen
;      All registers destroyed
; To restore background_sprite, load bc with (x, y) and call Restore_back (only!)
; =======================================================================================================
Get_back16: 
    push bc
    ld d, c
    ld e, b                 ; Findpixel need e = x and d = y
    call FindPixel          ; now hl is the start of diplay memory
    ld de, background_sprite      
    ex de, hl               ; de = display memory / hl = background_sprite adress
    ld b, 16                ; with djnz, we're now ready for 16 loop
Mcopy_loop2:
    push bc
    ld a, (de)              ; get the first byte of the row on the screen in a
    ld (hl), a              ; save it in the first byte at the same row in temp_sprite
    inc de                  ; ready to read the next byte on the screen
    inc hl                  ; ready to write the second byte in temp_sprite
    ld a, (de)              ; get the second byte of the row in a
    ld (hl), a              ; save it in the second byte at the same row in temp_sprite
    inc de                  ; de point to the next row of your sprite
    inc hl                  ; ready to write the third byte of virtual_sprite
    ld a, (de)              ; get the third byte of the row on the screen in a
    ld (hl), a              ; save it in the third byte at the same row in temp_sprite
    inc hl                  ; ready to write the next row in temp_sprite
    ex de, hl               ; we can not add a number to de, so we exange it with hl, and we recover hl after
    ld bc, 14
    add hl, bc              ; add 14 to hl
    ex de, hl               ; Eventually, add 14 to de: de point at the next row on the screen, hl is recovered
    pop bc                  ; restore counter value (b)
    djnz Mcopy_loop2        ; dec b and if b=0, then the sprite is finished copied (16 rows of data and 16 rows of mask) 
    pop bc
    
    ld d, $FF
    ld e, $00
    ld a, %00000111
    and b                   ; we get the 3 last bits of b : the x position on the grid
    jr z, Mno_prep          ; if the three last bits of b are 000, no preparation
    ld b, a                 ; laod b with this value for djnz
Mprep_slide:
    srl d                   ; we slide $FF one time on the right, bit 7 is reset
    rr e                    ; we slide $00 one time on the right, bit 7 is set
    djnz Mprep_slide        ; we make it b times
Mno_prep:                   ; now we are gonna make the borders
    ld b, 16                ; 16 rows
    ld hl, background_sprite
Mmake_borders
    ld a, d                 ; hl is on the first byte of the row (3 bytes)
    and (hl)                ; we cut what is on the left of the sprite zone (from 0 to 7 bits)
    ld (hl), a              ; save changes
    inc hl
    inc hl                  ; now hl is on the third byte of the row
    ld a, e
    and (hl)                ; we cut what is on the right of the sprite zone (from 0 to 8 bits)
    ld (hl), a              ; save changes
    inc hl                  ; to the next row
    djnz Mmake_borders      ; we make this 16 times (16 rows)

    ld b, 16                
    ld hl, background_mask
Mmake_mask
    ld (hl), d              ; first byte on the row
    inc hl
    ld (hl), $FF            ; second byte on the row
    inc hl
    ld (hl), e              ; third byte on the row
    inc hl
    djnz Mmake_mask
    ret                     ; now sprite preparation is finished
                            ; to restore it, use Restore back

Restore_back:
    ld d, c
    ld e, b                 ; Findpixel need e = x and d = y
    call FindPixel          ; now hl is the start of diplay memory
    ld de, background_sprite;     de is the start of the background_sprite
    jp MRestore_mid         ; we jump in the PutSprite16Masked routine

background_sprite:          ; background_sprite is a 24*16 sprite
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
background_mask:
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0
.db 0, 0, 0, 0, 0, 0

; * End of the routine code * (don't forget virtual_sprite, background_sprite and FindPixel routine)
; =======================================================================================================

; =============================================================
; Dan Eble & James Yopp FindPixel routine
; Input:  D = y, E = x
; Output: HL= addr in vid mem, A = bitmask, C is modified
; =============================================================
FindPixel:
        ld hl,FP_Bits
        ld a,e
        and $07         ; a = bit offset
        add a,l
        ld l,a
        adc a,h
        sub l
        ld h,a
        ld c,(hl)       ; c = bitmask for (hl)
                        ;48 t-states up to this point
        ld hl,FP_RLD
        ld (hl),d
        ld a,e          ; a = x/8 (byte offset within row)
        rrca
        rrca   
        rrca
        rld
        or $FC 
        ld l,(hl)
        ld h,a          ; hl -> byte in vid mem
        ld a,c          ; now a = bitmask for (hl)
                        ; 121 t-states up to this point
        ret

FP_RLD:  .db $00
FP_Bits: .db $80,$40,$20,$10,$08,$04,$02,$01

.end
